# @summary Installs the CIS Assessor along with required dependencies.
#
# CIS CAT Pro scanner requires a JRE as a prerequisite.
#
# @param allow_insecure
#   Enables skipping of certificate validation when the scanner_source parameter is specified.
#
#
#   This parameter should be used only with the scanner_source parameter.
#
# @param linux_manage_unzip
#   Determines if this module should manage the installation of unzip on Linux
#
# @param manage_java
#   Determines if this module should manage the installation of java
#   Note: Setting this to true on OSX nodes is not supported.
#
# @param scanner_source
#   The URL to the location of a specific Assessor distribution file.
#
#   This may be specified as a source attribute of a [file resource](https://puppet.com/docs/puppet/latest/types/file.html#file-attribute-source)
#
#   In the Puppet Comply application config, if you specified "We want to access the assessor through our ingress controller",
#   then set this value to https://`PE TLS FQDN`/assessor, otherwise set this value to https://`Comply Hostname`:30303/assessor
#
# @param scanner_version
#   Specify the version of the Assessor archive provided in `scanner_source`.
#   Defaults to the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param scanner_checksum
#   Specify the sha256 checksum of the Assessor archive provided in `scanner_source`.
#   Defaults to the checksum of the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param use_mtls
#   Use mutual TLS authentication. Comply is configured to only accept requests from nodes with valid certificates issued with the same
#   CA cert configured in Replicated. Ensure you have configured Comply properly using certs generated on your PE instance.
#
# @param windows_manage_wget
#   Allow Comply to manage the wget package. Required for fetching the Assessor.
#
# @example Mange packages elsewhere on Linux
#   class { 'comply::scanners::ciscat':
#     scanner_source     => 'https://comply.10.10.10.10.nip.io/assessor'
#     linux_manage_unzip => false,
#     manage_java        => false,
#   }
#
# @example Install the CIS Assessor when Comply is configured to access the assessor through an ingress controller
#   class { 'comply::scanners::ciscat':
#     scanner_source     => 'https://pe-tls-fqdn.10.10.10.10.nip.io/assessor'
#     linux_manage_unzip => false,
#     manage_java        => false,
#   }
#
# @example Install the CIS Assessor when Comply is configured to access the assessor through a NodePort
#   class { 'comply::scanners::ciscat':
#     scanner_source     => 'https://comply.10.10.10.10.nip.io:30303/assessor'
#     linux_manage_unzip => false,
#     manage_java        => false,
#   }
#
class comply::scanners::ciscat(
  Boolean $allow_insecure                             = $comply::allow_insecure,
  Boolean $linux_manage_unzip                         = $comply::linux_manage_unzip,
  Boolean $win_manage_7zip                            = $comply::win_manage_7zip,
  Boolean $manage_chocolatey                          = $comply::manage_chocolatey,
  Boolean $manage_java                                = $comply::manage_java,
  Optional[Stdlib::Filesource] $scanner_source        = $comply::scanner_source,
  Optional[Pattern[/\d+\.\d+\.\d+/]] $scanner_version = $comply::scanner_version,
  Optional[String] $scanner_checksum                  = $comply::scanner_checksum,
  Boolean $use_mtls                                   = $comply::use_mtls,
  Boolean $windows_manage_wget                        = $comply::windows_manage_wget,
) inherits comply {

  if $facts['kernel'] == 'windows' {
    $install_path = 'C:/ProgramData/PuppetLabs/comply'
    $archive_provider = 'wget'
  } else {
    $install_path = '/opt/puppetlabs/comply'
    $archive_provider = 'curl'
  }

  if $manage_chocolatey and $facts['kernel'] == 'windows' {
    include chocolatey
  }

  if $manage_java {
    if $facts['kernel'] == 'windows' {
      package { 'adoptopenjdk8jre':
        ensure   => latest,
        provider => 'chocolatey',
        install_options => ['-y'],
        notify   => Service['pxp-agent'],
      }
    } elsif $facts['kernel'] == 'Darwin' {
      fail('Setting manage_java => true is not supported on OSX. Please ensure a JRE is installed via alternative means.')
    } else {
      if ($facts['os']['family'] == 'RedHat') and ($facts['os']['release']['major'] == '6' or $facts['os']['release']['major'] == '7') {
        class { 'java':
          distribution => 'jre',
          package      => 'java-1.8.0-openjdk',
        }
      } else {
        class { 'java':
          distribution => 'jre',
        }
      }
    }
  }

  if $windows_manage_wget and $facts['kernel'] == 'windows' {
    package { 'wget':
      ensure   => latest,
      provider => 'chocolatey',
      install_options => ['-y'],
    }
  }

  if $linux_manage_unzip and $facts['kernel'] == 'Linux' {
    ensure_packages('unzip', {
      ensure => 'installed'
    })
  }

  if $win_manage_7zip and $facts['kernel'] == 'windows' {
    ensure_packages('7zip', {
      ensure   => 'installed',
      provider => 'chocolatey',
    })
  }

  if $use_mtls and $allow_insecure {
      fail('Cannot use mTLS and allow insecure together.')
  }

  $_download_options = $use_mtls ? {
    false => undef,
    true  => $facts['kernel'] ? {
      /(Darwin|Linux)/ => [
        '-E',
        "${facts['ssldir']}/certs/${trusted['certname']}.pem",
        '--key',
        "${facts['ssldir']}/private_keys/${trusted['certname']}.pem",
        '--cacert',
        "${facts['ssldir']}/certs/ca.pem"
      ],
      'windows'        => [
        '--certificate',
        "${facts['ssldir']}/certs/${trusted['certname']}.pem",
        '--private-key',
        "${facts['ssldir']}/private_keys/${trusted['certname']}.pem",
        '--ca-certificate',
        "${facts['ssldir']}/certs/ca.pem"
      ],
      default          => undef,
    }
  }

  file {
    default:
      ensure => directory,
      before => Archive["Assessor-CLI-${scanner_version}.zip"],
    ;
    $install_path: ;
    "${install_path}/assessors": ;
    "${install_path}/assessors/${scanner_version}": ;
    "${install_path}/tmp": ;
  }

  archive { "Assessor-CLI-${scanner_version}.zip":
    provider         => $archive_provider,
    allow_insecure   => $allow_insecure,
    cleanup          => true,
    creates          => "${install_path}/assessors/${scanner_version}/Assessor-CLI/Assessor-CLI.jar",
    download_options => $_download_options,
    extract          => true,
    extract_path     => "${install_path}/assessors/${scanner_version}",
    path             => "${install_path}/tmp/Assessor-CLI-${scanner_version}.zip",
    source           => $scanner_source,
    checksum_type    => 'sha256',
    checksum         => $scanner_checksum,
    checksum_verify  => true,
  }

  file {
    default:
      require => Archive["Assessor-CLI-${scanner_version}.zip"],
    ;
    "${install_path}/Assessor-CLI":
      ensure => link,
      force  => true,
      target => "${install_path}/assessors/${scanner_version}/Assessor-CLI",
    ;
    "${install_path}/assessors/${scanner_version}/Assessor-CLI/Assessor-CLI.jar":
      mode => '0755',
    ;
  }
}
